/* eslint-disable no-console */
const { execSync } = require('child_process')
const path = require('path')
const initializeHusky = require('./HuskyCommandsExecution/initializeHusky')
const configurePrecommitHook = require('./HuskyCommandsExecution/configurePrecommitHook')
const getNodeModulesPath = require('../../utils/getNodeModulesPath')
const { executeSynchronizedCommands } = require('../../utils/executeSyncCommands')
const { Logger } = require('../../utils/Logger')
const { isrequireThere } = require('../../utils/getScripts')

let isCustomPrecommitConfigurationSuccessful = false

function setupHusky(){
    let isHuskyPackageInstalled = installHuskyPackage()
    if(isHuskyPackageInstalled){
        let isHuskyInitializedSuccessfully = initializeHusky()

        if(isHuskyInitializedSuccessfully){
            Logger.log(Logger.SUCCESS_TYPE,'Husky installation successful')
            
            isCustomPrecommitConfigurationSuccessful = configurePrecommitHook()
            
            if(isCustomPrecommitConfigurationSuccessful){
                Logger.log(Logger.SUCCESS_TYPE,'Custom pre commit hook setup was successful')
                return true
            }
            else{
                Logger.log(Logger.FAILURE_TYPE,'Couldn\'t configure custom pre commit hook')
                return false
            }
        }
        else{
            return false
        }
    }
    else{
        Logger.log(Logger.FAILURE_TYPE,'Some issue in installing husky package.')
        return false
    }
}
function installHuskyPackage(){
    const packageJsonPath = path.join(getNodeModulesPath(),'package.json')

    let packageJsonContent = require(packageJsonPath)
    let pluginsIndevDependencies = packageJsonContent.devDependencies
    let pluginNamesOfDevDependencyPlugins = []
    let husky = {
        packageName: 'husky',
        version: '^7.0.4'
    }

    if(pluginsIndevDependencies !== undefined){
        pluginNamesOfDevDependencyPlugins = Object.keys(pluginsIndevDependencies)
    }
    let isHuskyInstalled = pluginNamesOfDevDependencyPlugins.some(pluginName => {
        if(pluginName === husky.packageName && pluginsIndevDependencies[pluginName] === husky.version && isrequireThere('husky')){
            return true
        }
    })

    if(!isHuskyInstalled){
        let installCommand = `npm install ${husky.packageName}@${husky.version} --save-dev`
        let isHuskyPackageInstalled = executeSynchronizedCommands(execSync,[installCommand],'Husky package Installed Successfully','Unable to install husky package',false,true)

        return isHuskyPackageInstalled
    }
    Logger.log(Logger.SUCCESS_TYPE,'Husky package already installed!')
    return true
}

module.exports = {
    setupHusky
} 

