const path = require('path')
const { writeFileSync, existsSync, unlinkSync, readFileSync } = require('fs')
const { getRepoName } = require('../pre-commit/gitSetup/gitUtils/gitActions.js')
const { getLibraryInstalledLocation } = require('./getLibraryInstalledLocation.js')
const getNodeModulesPath = require('./getNodeModulesPath.js')
const repoName = getRepoName()
const libraryInstalledLocation = getLibraryInstalledLocation()
const { executeSynchronizedCommands } = require('./executeSyncCommands.js')

let eslintConfiguration = ``
const eslintConfigFilePath = path.join(getNodeModulesPath(),'.eslintrc.js')

function createEslintConfigFile(){
    let pathOfServiceSpecificEslintConfigFile = path.join(libraryInstalledLocation,'configuration','services',repoName,'.eslintrc.js')
    eslintConfiguration = executeSynchronizedCommands(readFileSync,[pathOfServiceSpecificEslintConfigFile,'utf-8'],'','Unable to read content of eslint config file.',true,false)

    let isEslintConfigFilePresent = executeSynchronizedCommands(existsSync,[eslintConfigFilePath],'','',false,true)
    if(isEslintConfigFilePresent){
        executeSynchronizedCommands(unlinkSync,[eslintConfigFilePath],'Eslint configuration file removed successfully!','Unable to remove the eslint config file.',false,false)
        return createEslintConfigurationFile()
    }
    else if(!isEslintConfigFilePresent){
        return createEslintConfigurationFile()
    }
}

function createEslintConfigurationFile(){
    let isEslintConfigFileCreated = executeSynchronizedCommands(writeFileSync,[eslintConfigFilePath,eslintConfiguration,'utf-8'],'Eslint configuration file created successfully!','Unable to create and write an eslint configuration file.',false,true)
    return isEslintConfigFileCreated
}

module.exports = {
    createEslintConfigFile
}






