const fs = require('fs')
const {execSync} = require('child_process')
const {writeFileSync} = require('fs')
const path = require('path')
const getNodeModulesPath = require('./getNodeModulesPath')
const { Logger } = require('./Logger');
const { executeSynchronizedCommands } = require('./executeSyncCommands')
const { getRepoName } = require('../pre-commit/gitSetup/gitUtils/gitActions')
const { getLibraryInstalledLocation } = require('./getLibraryInstalledLocation')
const {repoURL} = require('../preCommitRequestUtils.json')
const { handleExceptionCase } = require('./getScripts')
const nodeModulesPathOfProject = getNodeModulesPath()

async function installPlugins(){
    const repoName = getRepoName()
    const serviceSpecificPluginsFile = path.join(getLibraryInstalledLocation(),'configuration','services',repoName,'pluginVersion.js')
    const pluginsFromCommonRepo = await handleExceptionCase([],`Kindly Ensure that Configuration is setup in Configuration repo \n ${repoURL}`,require,serviceSpecificPluginsFile);
    
    if(pluginsFromCommonRepo === undefined){
        return false
    }
    let packageJsonFilePath = path.join(nodeModulesPathOfProject,'package.json')
    let uninstalledPlugins = []
    let pluginsToBeAddedInDevDependencies = []
    let packageJsonContent = require(packageJsonFilePath)
    let pluginsIndevDependencies = packageJsonContent.devDependencies
    let pluginNamesOfDevDependencyPlugins = []
    let arePluginsPresentIndevDependencies = false
    let areRepoSpecificPluginsPresent = false
    let arePluginsInstalledSuccessfully = false
    let isPackageJsonChanged = false
    let isPackageJsonChangesRevertedSuccessfully = false

    if(pluginsIndevDependencies !== undefined){
        arePluginsPresentIndevDependencies = true
        pluginNamesOfDevDependencyPlugins = Object.keys(pluginsIndevDependencies)
    }
    else if(pluginsIndevDependencies === undefined){
        Logger.log(Logger.INFO_TYPE,'No devDependencies present!')
    }

    areRepoSpecificPluginsPresent = pluginsFromCommonRepo.length !== 0 ? true : false

    if(areRepoSpecificPluginsPresent || arePluginsPresentIndevDependencies){
        pluginsFromCommonRepo.map(plugin => {
            if(! pluginNamesOfDevDependencyPlugins.includes(plugin.packageName)){
                let pluginToBeInstalled = `${plugin.packageName}@${plugin.version}`
                uninstalledPlugins.push(pluginToBeInstalled)
            }
            else if(pluginNamesOfDevDependencyPlugins.includes(plugin.packageName)){
                if(plugin.version !== pluginsIndevDependencies[plugin.packageName]){
                    let pluginToBeInstalled = `${plugin.packageName}@${plugin.version}`
                    uninstalledPlugins.push(pluginToBeInstalled)
                }
            }
        })
                
        if(uninstalledPlugins.length > 0){
            executeSynchronizedCommands(process.chdir,[nodeModulesPathOfProject],'','Unable to navigate to node_modules path of project while trying to install plugins',false,false)
            let installCommand = `npm install --save-dev ${uninstalledPlugins.join(' ')}`
            console.log('installCommand: ',installCommand)
            arePluginsInstalledSuccessfully = executeSynchronizedCommands(execSync,[installCommand,{stdio:'inherit'}],'Plugins are installed successfully','Unable to install the plugins',false,true)
            if(arePluginsInstalledSuccessfully){
                executeSynchronizedCommands(process.chdir,[nodeModulesPathOfProject],'','Unable to navigate to node_modules path when trying to revert changes in package.json',false,false)
                let gitDiffOutput = executeSynchronizedCommands(execSync,['git diff --name-only'],'','Unable to execute git diff command while checking if package.json is changed',true,false).toString().trim()
                isPackageJsonChanged = gitDiffOutput.includes('package.json') ? true : false
                if(isPackageJsonChanged){
                    let packageJsonChangesRevertCommand = `git checkout "${packageJsonFilePath}"`
                    isPackageJsonChangesRevertedSuccessfully = executeSynchronizedCommands(execSync,[packageJsonChangesRevertCommand,{stdio:'inherit'}],'Changes in package.json reverted successfully','Unable to revert the changes in package.json file',false,true)
                        
                    if(isPackageJsonChangesRevertedSuccessfully){
                        uninstalledPlugins.map(uninstalledPlugin => {
                            if(uninstalledPlugin.startsWith('@')){
                                let indexOfAtCharacter = uninstalledPlugin.indexOf('@')
                                let indexOfSecondOccurenceOfAtCharacter = uninstalledPlugin.indexOf('@',indexOfAtCharacter + 1)
                                let unInstalledPluginName = uninstalledPlugin.slice(0,indexOfSecondOccurenceOfAtCharacter)
                                let unInstalledPluginVersion = uninstalledPlugin.slice(indexOfSecondOccurenceOfAtCharacter + 1)
                                pluginsToBeAddedInDevDependencies[unInstalledPluginName] = unInstalledPluginVersion
                            }
                            else{
                                pluginsToBeAddedInDevDependencies[uninstalledPlugin.split('@')[0]] = uninstalledPlugin.split('@')[1]
                            }
                            })
                            let updatedPluginsIndevDependencies = {
                                ...pluginsIndevDependencies,
                                ...pluginsToBeAddedInDevDependencies
                            }
                            packageJsonContent.devDependencies = updatedPluginsIndevDependencies
                            
                            let modifiedPackageJson = {
                                ...packageJsonContent,
                            }
                            let isPluginsAddedInPackageJson = executeSynchronizedCommands(writeFileSync,[packageJsonFilePath,JSON.stringify(modifiedPackageJson,null,2)],'Newly installed plugins successfully added to package.json','Unable to append installed plugins to package.json',false,true)
                            return isPluginsAddedInPackageJson 
                    }
                    else{
                        Logger.log(Logger.SUCCESS_TYPE,'package.json not changed')
                        return true
                    }
                }
                else{
                    return false
                }
            }
            else if(uninstalledPlugins.length === 0){
                Logger.log(Logger.INFO_TYPE,'Plugins are installed already')
                arePluginsInstalledSuccessfully = true
                return arePluginsInstalledSuccessfully
            }
        }
        else{
            Logger.log(Logger.INFO_TYPE,'No plugins present for the repository.')
            return true
        }
    }   
}


module.exports = {
    installPlugins
}
